@@ -10,6 +10,7 @@
 //!
 //! assert_eq!(hand.as_str(), "\u{1f90c}");
 //! assert_eq!(hand.name(), "pinched fingers");
+//! assert_eq!(hand.unicode_version(), emojis::UnicodeVersion::new(13, 0));
 //! assert_eq!(hand.group(), emojis::Group::PeopleAndBody);
 //! assert_eq!(hand.shortcode().unwrap(), "pinched_fingers");
 //! assert_eq!(hand.skin_tone().unwrap(), emojis::SkinTone::Default);
@@ -21,6 +22,14 @@
 //! assert_eq!(smiley, "😀");
 //! ```
 //!
+//! Iterate and filter out newer emoji versions.
+//! ```
+//! let iter = emojis::iter().filter(|e| {
+//!     e.unicode_version() < emojis::UnicodeVersion::new(13, 0)
+//! });
+//! assert_eq!(iter.count(), 1738);
+//! ```
+//!
 //! Iterate over all the emojis in a group.
 //! ```
 //! let grapes = emojis::Group::FoodAndDrink.emojis().next().unwrap();
@@ -43,9 +52,9 @@ extern crate alloc;
 
 mod generated;
 
-use core::cmp;
 use core::convert;
 use core::fmt;
+use core::{cmp, hash};
 
 pub use crate::generated::Group;
 
@@ -53,17 +62,24 @@ pub use crate::generated::Group;
 ///
 /// See [Unicode.org](https://unicode.org/emoji/charts/full-emoji-list.html) for
 /// more information.
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+#[derive(Debug)]
 pub struct Emoji {
-    id: u16,
     emoji: &'static str,
     name: &'static str,
+    unicode_version: UnicodeVersion,
     group: Group,
     skin_tone: Option<(u16, SkinTone)>,
     aliases: Option<&'static [&'static str]>,
     variations: &'static [&'static str],
 }
 
+/// Represents a Unicode version.
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, PartialOrd, Ord)]
+pub struct UnicodeVersion {
+    major: u32,
+    minor: u32,
+}
+
 /// Represents the skin tone of an emoji.
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum SkinTone {
@@ -75,6 +91,21 @@ pub enum SkinTone {
     Dark,
 }
 
+impl UnicodeVersion {
+    /// Construct a new version.
+    pub const fn new(major: u32, minor: u32) -> Self {
+        Self { major, minor }
+    }
+
+    pub const fn major(self) -> u32 {
+        self.major
+    }
+
+    pub const fn minor(self) -> u32 {
+        self.minor
+    }
+}
+
 impl Emoji {
     /// Returns this emoji as a string.
     ///
@@ -100,13 +131,27 @@ impl Emoji {
         self.name
     }
 
+    /// Returns the Unicode version this emoji first appeared in.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use emojis::UnicodeVersion;
+    ///
+    /// let villain = emojis::lookup("🦹").unwrap();
+    /// assert_eq!(villain.unicode_version(), UnicodeVersion::new(11, 0));
+    /// ```
+    pub const fn unicode_version(&self) -> UnicodeVersion {
+        self.unicode_version
+    }
+
     /// Returns this emoji's group.
     ///
     /// # Examples
     ///
     /// ```
-    /// # use emojis::Group;
-    /// #
+    /// use emojis::Group;
+    ///
     /// let flag = emojis::lookup("🇿🇦").unwrap();
     /// assert_eq!(flag.group(), Group::Flags);
     /// ```
@@ -157,8 +202,8 @@ impl Emoji {
     /// assert!(cool.skin_tones().is_none());
     /// ```
     pub fn skin_tones(&self) -> Option<impl Iterator<Item = &'static Self>> {
-        let (id, _) = self.skin_tone?;
-        Some(crate::generated::EMOJIS[id as usize..].iter().take(6))
+        let (i, _) = self.skin_tone?;
+        Some(crate::generated::EMOJIS[i as usize..].iter().take(6))
     }
 
     /// Returns a version of this emoji that has the given skin tone.
@@ -205,15 +250,29 @@ impl Emoji {
     }
 }
 
+impl cmp::PartialEq<Emoji> for Emoji {
+    fn eq(&self, other: &Emoji) -> bool {
+        self.emoji == other.emoji
+    }
+}
+
 impl cmp::PartialEq<str> for Emoji {
     fn eq(&self, s: &str) -> bool {
-        cmp::PartialEq::eq(self.as_str(), s)
+        self.as_str() == s
     }
 }
 
 impl cmp::PartialEq<&str> for Emoji {
     fn eq(&self, s: &&str) -> bool {
-        cmp::PartialEq::eq(self.as_str(), *s)
+        self.as_str() == *s
+    }
+}
+
+impl cmp::Eq for Emoji {}
+
+impl hash::Hash for Emoji {
+    fn hash<H: hash::Hasher>(&self, state: &mut H) {
+        self.emoji.hash(state);
     }
 }
 
@@ -225,7 +284,7 @@ impl convert::AsRef<str> for Emoji {
 
 impl fmt::Display for Emoji {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        fmt::Display::fmt(self.as_str(), f)
+        self.as_str().fmt(f)
     }
 }
 
@@ -302,6 +361,16 @@ mod tests {
         assert_eq!(buf.as_str(), "😀");
     }
 
+    #[test]
+    fn version_ordering() {
+        assert!(UnicodeVersion::new(13, 0) >= UnicodeVersion::new(12, 0));
+        assert!(UnicodeVersion::new(12, 1) >= UnicodeVersion::new(12, 0));
+        assert!(UnicodeVersion::new(12, 0) >= UnicodeVersion::new(12, 0));
+        assert!(UnicodeVersion::new(12, 0) < UnicodeVersion::new(12, 1));
+        assert!(UnicodeVersion::new(11, 0) < UnicodeVersion::new(12, 1));
+        assert!(UnicodeVersion::new(11, 0) < UnicodeVersion::new(12, 1));
+    }
+
     #[test]
     fn lookup_variation() {
         assert_eq!(lookup("☹"), lookup("☹️"));
